<?php

namespace App\Filament\Resources;

use App\Enums\ModalityEnum;
use App\Filament\Forms\Components\PtbrMoney;
use App\Filament\Resources\ApolicyResource\Pages;
use App\Models\Apolicy;
use Carbon\Carbon;
use Filament\Forms;
use Filament\Forms\Form;
use Filament\Resources\Resource;
use Filament\Tables;
use Filament\Tables\Table;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\SoftDeletingScope;

class ApolicyResource extends Resource
{
    protected static ?string $model = Apolicy::class;

    protected static ?string $navigationIcon = 'phosphor-files-duotone';

    protected static ?string $modelLabel = 'Apolice';

    protected static bool $isScopedToTenant = false;

    public static function form(Form $form): Form
    {
        return $form
            ->schema([
                Forms\Components\Section::make()
                    ->columns(6)
                    ->schema([
                        Forms\Components\Grid::make(4)
                            ->schema([
                                Forms\Components\DatePicker::make('date_request')
                                    ->label('Data da solicitação')
                                    ->columnSpan(1)
                                    ->default(Carbon::now())
                                    ->required(),
                                Forms\Components\Select::make('registry_id')
                                    ->label('Tomador')
                                    ->required()
                                    ->relationship('registry', 'name')
                                    ->searchable()
                                    ->columnSpan(3)
                                    ->createOptionAction(fn() => \App\Filament\Resources\RegistryResource::getUrl('index')),
                            ]),
                        Forms\Components\Grid::make(5)
                            ->schema([
                                Forms\Components\TextInput::make('insured')
                                    ->label('Segurado')
                                    ->placeholder('Informe o nome do segurado')
                                    ->required()
                                    ->columnSpan(2)
                                    ->maxLength(255),
                                Forms\Components\TextInput::make('modality_group')
                                    ->label('Grupo de modalidade')
                                    ->placeholder('Informe o grupo de modalidade')
                                    ->required()
                                    ->maxLength(255),
                                Forms\Components\Select::make('modality_id')
                                    ->label('Modalidade')
                                    ->required()
                                    ->options(ModalityEnum::class)
                                    ->searchable()
                                    ->reactive(),
                                PtbrMoney::make('insured_value')
                                    ->label('Importância segurada')
                                    ->placeholder('Informe a Valor')
                                    ->live()
                                    ->required()
                                    ->prefix('R$')
                                    ->reactive(),
                            ]),
                        Forms\Components\Grid::make(12)
                            ->schema([
                                Forms\Components\DatePicker::make('start_validity')
                                    ->label('Inicio de Vigência')
                                    ->columnSpan(2)
                                    ->live()
                                    ->required(),
                                Forms\Components\TextInput::make('validity')
                                    ->label('Validade')
                                    ->placeholder('Informe a validade')
                                    ->required()
                                    ->live()
                                    ->afterStateUpdated(function (Forms\Set $set, Forms\Get $get, $state) {
                                        $startValidity = $get('start_validity');

                                        if ($startValidity && $state) {
                                            $endValidity = Carbon::parse($startValidity)->addDays((int) $state)->format('Y-m-d');
                                            $set('end_validity', $endValidity);
                                        }
                                    })
                                    ->columnSpan(1)
                                    ->numeric(),
                                Forms\Components\DatePicker::make('end_validity')
                                    ->label('Fim de Vigência')
                                    ->columnSpan(2)
                                    ->readOnly()
                                    ->live()
                                    ->required(),
                                Forms\Components\TextInput::make('contract_name')
                                    ->label('Nome do Edital/Contrato')
                                    ->placeholder('Informe o nome do edital/contrato')
                                    ->required()
                                    ->columnSpan(4)
                                    ->maxLength(200),
                                Forms\Components\TextInput::make('contract_number')
                                    ->label('Numero do Edital/Contrato')
                                    ->placeholder('Informe o numero do edital/contrato')
                                    ->required()
                                    ->columnSpan(3)
                                    ->maxLength(200),
                            ]),
                        Forms\Components\Grid::make(12)
                            ->schema([
                                Forms\Components\TextInput::make('contract_objective')
                                    ->label('Objetivo Contrato')
                                    ->placeholder('Informe o objetivo do contrato')
                                    ->required()
                                    ->columnSpan(5)
                                    ->maxLength(200),
                                Forms\Components\TextInput::make('special_conditions')
                                    ->label('Condições Particulares')
                                    ->placeholder('Informe as condições particulares')
                                    ->required()
                                    ->columnSpan(5)
                                    ->maxLength(200),
                                Forms\Components\TextInput::make('installments')
                                    ->label('Parcelas')
                                    ->placeholder('Informe a quantidade de parcelas')
                                    ->required()
                                    ->columnSpan(1)
                                    ->numeric(),
                                Forms\Components\TextInput::make('days')
                                    ->required()
                                    ->label('Dias')
                                    ->placeholder('Informe a quantidade de dias')
                                    ->columnSpan(1)
                                    ->numeric(),
                            ]),
                        Forms\Components\Textarea::make('description')
                            ->label('Descrição')
                            ->placeholder('Informe a descrição')
                            ->columnSpanFull(),
                        Forms\Components\Toggle::make('is_active')
                            ->label('Status')
                            ->default(true)
                            ->hiddenOn(['create'])
                            ->required(),
                    ]),
            ]);
    }

    public static function table(Table $table): Table
    {
        return $table
            ->emptyStateHeading('Apolice não encontrados!')
            ->emptyStateDescription('Para adicionar uma apolice clique no botão abaixo.')
            ->emptyStateActions([
                Tables\Actions\CreateAction::make(),
            ])
            ->columns([
                Tables\Columns\TextColumn::make('date_request')
                    ->label('Data da solicitação')
                    ->date('d/m/Y')
                    ->sortable(),
                Tables\Columns\TextColumn::make('registry.name')
                    ->label('Tomador')
                    ->sortable(),
                Tables\Columns\IconColumn::make('is_active')
                    ->label('Status')
                    ->boolean(),
                Tables\Columns\TextColumn::make('created_at')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('updated_at')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('deleted_at')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
            ])
            ->filters([
                Tables\Filters\TrashedFilter::make(),
            ])
            ->actions([
                Tables\Actions\ViewAction::make(),
                Tables\Actions\EditAction::make(),
                Tables\Actions\Action::make('pdf')
                    ->icon('heroicon-o-document')
                    ->label('PDF')
                    ->url(
                        fn (Apolicy $apolicy): string => route('report.apolicy', $apolicy->uuid),
                        shouldOpenInNewTab: true
                    )
//                    ->visible(fn(): bool => auth()->user()->can('pdf_receipt::payment'))
                    ->openUrlInNewTab(),
            ])
            ->bulkActions([
                Tables\Actions\BulkActionGroup::make([
                    Tables\Actions\DeleteBulkAction::make(),
                    Tables\Actions\ForceDeleteBulkAction::make(),
                    Tables\Actions\RestoreBulkAction::make(),
                ]),
            ]);
    }

    public static function getRelations(): array
    {
        return [
            //
        ];
    }

    public static function getPages(): array
    {
        return [
            'index'  => Pages\ListApolicies::route('/'),
            'create' => Pages\CreateApolicy::route('/create'),
            'view'   => Pages\ViewApolicy::route('/{record}'),
            'edit'   => Pages\EditApolicy::route('/{record}/edit'),
        ];
    }

    public static function getEloquentQuery(): Builder
    {
        return parent::getEloquentQuery()
            ->withoutGlobalScopes([
                SoftDeletingScope::class,
            ]);
    }
}
