<?php

namespace App\Filament\Resources;

use App\Enums\StateEnum;
use App\Filament\Resources\CityResource\Pages;
use App\Models\City;
use BezhanSalleh\FilamentShield\Contracts\HasShieldPermissions;
use Filament\Forms;
use Filament\Forms\Form;
use Filament\Resources\Resource;
use Filament\Tables;
use Filament\Tables\Grouping\Group;
use Filament\Tables\Table;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\SoftDeletingScope;
use Illuminate\Support\Str;

class CityResource extends Resource implements HasShieldPermissions
{
    protected static ?string $model = City::class;

    protected static ?string $navigationIcon = 'phosphor-city-duotone';

    protected static ?string $navigationGroup = 'Cadastros';

    protected static ?string $pluralLabel = 'Cidades';
    protected static ?string $modelLabel = 'Cidade';
    protected static bool $isScopedToTenant = false;

    public static function form(Form $form): Form
    {
        return $form
            ->schema([
                Forms\Components\Grid::make(4)
            ->schema([
                Forms\Components\TextInput::make('name')
                    ->label('Nome')
                    ->placeholder('Informe o nome da cidade')
                    ->required()
                    ->live()
                    ->columnSpanFull()
                    ->afterStateUpdated(fn ($set, $state) => $set('name', Str::upper($state)))
                    ->maxLength(200),
                Forms\Components\Select::make('state_id')
                    ->label('Estado')
                    ->required()
                    ->options(StateEnum::class)
                    ->searchable()
                    ->columnSpan(2)
                    ->reactive(),
                Forms\Components\TextInput::make('ibge_code')
                    ->label('Código IBGE')
                    ->placeholder('Informe o código IBGE da cidade')
                    ->required()
                    ->columnSpan(2)
                    ->maxLength(7),
                Forms\Components\Toggle::make('status')
                    ->inline(false)
                    ->hiddenOn(['create'])
                    ->label('Status')
                    ->columnSpanFull()
                    ->required(),
                ])
            ]);
    }

    public static function table(Table $table): Table
    {
        return $table
            ->deferLoading()
            ->columns([
                Tables\Columns\TextColumn::make('name')
                    ->label('Nome')
                    ->sortable()
                    ->searchable(),
                Tables\Columns\TextColumn::make('state')
                    ->label('Estado')
                    ->alignCenter()
                    ->sortable()
                    ->searchable(),
                Tables\Columns\TextColumn::make('ibge_code')
                    ->label('Código IBGE')
                    ->alignCenter()
                    ->sortable()
                    ->searchable(),
                Tables\Columns\IconColumn::make('status')
                    ->label('Status')
                    ->boolean(),
                Tables\Columns\TextColumn::make('created_at')
                    ->label('Criado em')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('updated_at')
                    ->label('Atualizado em')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('deleted_at')
                    ->label('Excluído em')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
            ])->paginated([10, 25, 50, 100])->striped()
            ->groups([
                Group::make('state')
                    ->label('Estado'),
            ])
            ->filters([
                Tables\Filters\TrashedFilter::make(),
                Tables\Filters\SelectFilter::make('state')
                    ->label('Estado')
                    ->options(StateEnum::class)
                    ->searchable(),
            ])
            ->actions([
                Tables\Actions\EditAction::make(),
                Tables\Actions\DeleteAction::make(),
                Tables\Actions\RestoreAction::make(),
            ])
            ->bulkActions([
                Tables\Actions\BulkActionGroup::make([
                    Tables\Actions\DeleteBulkAction::make(),
                    Tables\Actions\RestoreBulkAction::make(),
                ]),
            ]);
    }

    public static function getPages(): array
    {
        return [
            'index' => Pages\ManageCities::route('/'),
        ];
    }

    public static function getEloquentQuery(): Builder
    {
        return parent::getEloquentQuery()
            ->withoutGlobalScopes([
                SoftDeletingScope::class,
            ]);
    }

    public static function getPermissionPrefixes(): array
    {
        return [
            'view_any',
            'create',
            'update',
            'delete',
        ];
    }
}
