<?php

namespace App\Filament\Resources\CompanyResource\Forms\Tabs;

use App\Models\City;
use App\Models\Company;
use Filament\Facades\Filament;
use Filament\Forms\Components\Actions\Action;
use Filament\Forms\Components\DateTimePicker;
use Filament\Forms\Components\FileUpload;
use Filament\Forms\Components\Grid;
use Filament\Forms\Components\Section;
use Filament\Forms\Components\Select;
use Filament\Forms\Components\Textarea;
use Filament\Forms\Components\TextInput;
use Filament\Forms\Components\Toggle;
use Filament\Forms\Set;
use Illuminate\Support\Facades\Http;
use Leandrocfe\FilamentPtbrFormFields\Cep;
use Leandrocfe\FilamentPtbrFormFields\Document;
use Leandrocfe\FilamentPtbrFormFields\PhoneNumber;

class Data extends Grid
{
    protected function setUp(): void
    {
        parent::setUp();

        $this->schema([
            Section::make('')
                ->schema([
                    Grid::make('grid')
                        ->schema([
                            Document::make('document_number')
                                ->label('CPF/CNPJ')
                                ->placeholder('Informe o CPF/CNPJ')
                                ->required()
                                ->suffixAction(
                                    Action::make('copyCostToPrice')
                                        ->label('Buscar CNPJ')
                                        ->icon('heroicon-o-magnifying-glass')
                                        ->requiresConfirmation()
                                        ->action(function (Set $set, $livewire, $state) {

                                            $state = preg_replace('/[^0-9]/', '', $state);
                                            $livewire->validateOnly('data.cpfcnpj');

                                            if (strlen($state) == 14) {

                                                $document = Http::get("https://publica.cnpj.ws/cnpj/$state")->throw()->json();

                                                if (in_array('erro', $document)) {
                                                    throw ValidationException::withMessages([
                                                        'data.cpfcnpj' => 'CNPJ inválido',
                                                    ]);
                                                }

                                                $set('name', $document['razao_social']);
                                                $set('state_registration', $document['estabelecimento']['inscricoes_estaduais'][0]['inscricao_estadual'] ?? '');
                                                $set('fantasy_name', $document['estabelecimento']['nome_fantasia']);
                                                $set('cep', zip_code_view($document['estabelecimento']['cep']));
                                                $set('address', $document['estabelecimento']['logradouro']);
                                                $set('number', $document['estabelecimento']['numero']);
                                                $set('supplements', $document['estabelecimento']['complemento']);
                                                $set('district', $document['estabelecimento']['bairro']);
                                                $set('phone2', $document['estabelecimento']['telefone1']);
                                                $set('mobile_phone', $document['estabelecimento']['telefone2']);
                                                $ibge_id = $set('mobile_phone', $document['estabelecimento']['cidade']['ibge_id']);

                                                $set('city_id', City::where(['ibge_code' => $ibge_id])->first()?->id);
                                            }
                                        })
                                )
                                ->dynamic(),
                        ])->columns(3),
                    Grid::make('grid')
                        ->schema([
                            Select::make('company_id')
                                ->label('Matriz')
                                ->options(function () {
                                    return Company::whereNull('company_id')
                                        ->where('id', '!=', Filament::getTenant()->id)
                                        ->get()->map(function ($business) {
                                            return [
                                                'value' => $business->id,
                                                'label' => $business->name . ' - ' . $business->fantasy_name . ' (' . $business->document_number . ')',
                                            ];
                                        })->pluck('label', 'value');
                                })
                                ->getOptionLabelFromRecordUsing(function (Company $business): string {
                                    return "{$business->name} - {$business->id}";
                                })
                                ->columnSpan(2)
                                ->searchable()
                                ->preload(),
                        ])->columns(1),
                    Grid::make('grid')
                        ->schema([
                            TextInput::make('name')
                                ->label('Razão Social')
                                ->placeholder('Informe o nome')
                                ->required()
                                ->columnSpan(10)
                                ->maxLength(200),
                            TextInput::make('initial')
                                ->label('Sigla')
                                ->placeholder('Informe a sigla')
                                ->required()
                                ->columnSpan(2)
                                ->maxLength(15),
                        ])->columns(12),
                    Grid::make('grid')
                        ->schema([
                            TextInput::make('fantasy_name')
                                ->label('Nome Fantasia')
                                ->placeholder('Informe o nome fantasia')
                                ->required()
                                ->maxLength(100),
                            TextInput::make('state_registration')
                                ->label('Incrição Estadual')
                                ->placeholder('Informe o registro estadual')
                                ->maxLength(20),
                            TextInput::make('municipal_registration')
                                ->label('Incrição Municipal')
                                ->placeholder('Informe o registro municipal')
                                ->maxLength(20),
                        ])->columns(3),
                    Grid::make('grid')
                        ->schema([
                            Cep::make('zip_code')
                                ->label('CEP')
                                ->required()
                                ->placeholder('Informe o CEP')
                                ->viaCep(
                                    mode: 'suffix', // Determines whether the action should be appended to (suffix) or prepended to (prefix) the cep field, or not included at all (none).
                                    errorMessage: 'CEP inválido.', // Error message to display if the CEP is invalid.

                                    setFields: [
                                        'address'        => 'logradouro',
                                        'address_number' => 'numero',
                                        'supplements'    => 'complemento',
                                        'neighborhood'   => 'bairro',
                                    ]
                                ),
                            TextInput::make('address')
                                ->label('Endereço')
                                ->placeholder('Informe o endereço')
                                ->required()
                                ->columnSpan(3)
                                ->maxLength(255),
                            TextInput::make('address_number')
                                ->label('Número')
                                ->placeholder('Informe o número')
                                ->required()
                                ->maxLength(10),
                            TextInput::make('neighborhood')
                                ->label('Bairro')
                                ->placeholder('Informe o bairro')
                                ->required()
                                ->maxLength(100),
                            Select::make('city_id')
                                ->label('Cidade/Estado')
                                ->relationship('city', 'name')
                                ->getOptionLabelFromRecordUsing(fn (City $city): string => "{$city->name} ({$city->state})")
                                ->searchable()
                                ->columnSpan(2)
                                ->preload()
                                ->required(),
                            TextInput::make('address_complement')
                                ->label('Complemento')
                                ->placeholder('Informe o complemento')
                                ->columnSpanFull()
                                ->maxLength(200),
                        ])->columns(4),
                    Grid::make('grid')
                        ->schema([
                            PhoneNumber::make('mobile_number')
                                ->label('Celular')
                                ->placeholder('Informe o celular')
                                ->required(),
                            PhoneNumber::make('phone_number')
                                ->label('Telefone')
                                ->placeholder('Informe o telefone'),
                            TextInput::make('email')
                                ->label('E-mail')
                                ->placeholder('Informe o e-mail')
                                ->required()
                                ->email()
                                ->maxLength(255),
                        ])->columns(3),
                    Textarea::make('observation')
                        ->label('Observações')
                        ->placeholder('Informe as observações')
                        ->maxLength(65535)
                        ->columnSpanFull(),
                    Section::make()
            ->schema([
                FileUpload::make('certified')
                    ->label('Arquivo do certificado A1 (Formato .pfx ou .p12)')
                    ->directory('certificate')
                    ->visibility('private')
                    ->columnSpanFull(),
                TextInput::make('certified_password')
                    ->label('Senha')
                    ->placeholder('Senha do usuário')
                    ->password()
                    ->revealable()
                    ->autocomplete(false)
                    ->maxLength(50),
                DateTimePicker::make('certified_expiration_date')
                    ->label('Validade do Certificado')
                    ->placeholder('Data do certificado')
                    ->disabled(),
            ]),
                    Grid::make('grid')
                        ->schema([
                            Toggle::make('is_active')
                                ->label('Status')
                                ->inline(false)
                                ->default(true)
                                ->required(),
                        ])->columns(8),
                ]),
        ])->columns(1);
    }
}
