<?php

namespace App\Filament\Resources;

use App\Filament\Forms\Components\ZipCode;
use App\Filament\Resources\RegistryResource\Pages;
use App\Models\City;
use App\Models\Registry;
use Filament\Forms;
use Filament\Forms\Components\Actions\Action;
use Filament\Forms\Components\Grid;
use Filament\Forms\Components\Select;
use Filament\Forms\Components\TextInput;
use Filament\Forms\Form;
use Filament\Forms\Set;
use Filament\Resources\Resource;
use Filament\Tables;
use Filament\Tables\Table;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\SoftDeletingScope;
use Illuminate\Support\Facades\Http;
use Leandrocfe\FilamentPtbrFormFields\Document;
use Leandrocfe\FilamentPtbrFormFields\PhoneNumber;

class RegistryResource extends Resource
{
    protected static ?string $model = Registry::class;

    protected static ?string $navigationIcon = 'phosphor-users-three-duotone';

    protected static ?string $modelLabel = 'Cliente';

    protected static bool $isScopedToTenant = false;

    public static function form(Form $form): Form
    {
        return $form
            ->schema([
                Forms\Components\Section::make()
                    ->schema([

                        Grid::make('grid')
                            ->schema([
                                Document::make('document_number')
                                    ->label('CNPJ')
                                    ->placeholder('Informe o CNPJ')
                                    ->required()
                                    ->suffixAction(
                                        Action::make('copyCostToPrice')
                                            ->label('Buscar CNPJ')
                                            ->icon('heroicon-o-magnifying-glass')
                                            ->requiresConfirmation()
                                            ->action(function (Set $set, $livewire, $state) {

                                                $state = preg_replace('/[^0-9]/', '', $state);
                                                $livewire->validateOnly('data.cpfcnpj');

                                                if (strlen($state) == 14) {

                                                    $document = Http::get("https://publica.cnpj.ws/cnpj/$state")->throw()->json();

                                                    if (in_array('erro', $document)) {
                                                        throw ValidationException::withMessages([
                                                            'data.cpfcnpj' => 'CNPJ inválido',
                                                        ]);
                                                    }

                                                    $set('name', $document['razao_social']);
                                                    $set('state_registration', $document['estabelecimento']['inscricoes_estaduais'][0]['inscricao_estadual'] ?? '');
                                                    $set('fantasy_name', $document['estabelecimento']['nome_fantasia']);
                                                    $set('zip_code', zip_code_view($document['estabelecimento']['cep']));
                                                    $set('address', $document['estabelecimento']['logradouro']);
                                                    $set('address_number', $document['estabelecimento']['numero']);
                                                    $set('address_complement', $document['estabelecimento']['complemento']);
                                                    $set('neighborhood', $document['estabelecimento']['bairro']);
                                                    $set('phone_number', $document['estabelecimento']['telefone1']);
                                                    $set('mobile_phone', $document['estabelecimento']['telefone2']);

                                                    $ibge_id = $set('mobile_phone', $document['estabelecimento']['cidade']['ibge_id']);

                                                    $set('city_id', City::where(['ibge_code' => $ibge_id])->first()?->id);
                                                }
                                            })
                                    )
                                    ->columnSpan(3)
                                    ->cnpj(),
                                Forms\Components\TextInput::make('state_registration')
                                    ->label('Inscrição Estadual')
                                    ->placeholder('Informe a Inscrição Estadual')
                                    ->columnSpan(3)
                                    ->maxLength(30),
                            ])->columns(12),
                        Grid::make('grid')
                            ->schema([
                                Forms\Components\TextInput::make('name')
                                    ->label('Razão Social')
                                    ->placeholder('Informe a Razão Social')
                                    ->required()
                                    ->columnSpan(6)
                                    ->maxLength(255),
                                Forms\Components\TextInput::make('fantasy_name')
                                    ->label('Nome Fantasia')
                                    ->placeholder('Informe o Nome Fantasia')
                                    ->columnSpan(6)
                                    ->maxLength(255),
                            ])->columns(12),
                        Grid::make('grid')
                            ->schema([
                                ZipCode::make('zip_code')
                                    ->required()
                                    ->columnSpan(1)
                                    ->live(onBlur: true)
                                    ->viaCep(
                                        setFields: [
                                            'address'        => 'logradouro',
                                            'address_number' => 'numero',
                                            'supplements'    => 'complemento',
                                            'neighborhood'   => 'bairro',
                                        ]
                                    ),
                                TextInput::make('address')
                                    ->label('Endereço')
                                    ->placeholder('Informe o endereço')
                                    ->required()
                                    ->columnSpan(3)
                                    ->maxLength(255),
                                TextInput::make('address_number')
                                    ->label('Número')
                                    ->placeholder('Informe o número')
                                    ->required()
                                    ->maxLength(10),
                                TextInput::make('neighborhood')
                                    ->label('Bairro')
                                    ->placeholder('Informe o bairro')
                                    ->required()
                                    ->maxLength(100),
                                Select::make('city_id')
                                    ->label('Cidade/Estado')
                                    ->relationship('city', 'name')
                                    ->getOptionLabelFromRecordUsing(fn (City $city): string => "{$city->name} ({$city->state})")
                                    ->searchable()
                                    ->columnSpan(2)
                                    ->preload()
                                    ->required(),
                                TextInput::make('address_complement')
                                    ->label('Complemento')
                                    ->placeholder('Informe o complemento')
                                    ->columnSpanFull()
                                    ->maxLength(200),
                            ])->columns(4),
                        Grid::make('grid')
                            ->schema([
                                PhoneNumber::make('mobile_number')
                                    ->label('Celular')
                                    ->placeholder('Informe o celular')
                                    ->required(),
                                PhoneNumber::make('phone_number')
                                    ->label('Telefone')
                                    ->placeholder('Informe o telefone'),
                                TextInput::make('email')
                                    ->label('E-mail')
                                    ->placeholder('Informe o e-mail')
                                    ->required()
                                    ->email()
                                    ->maxLength(255),
                                TextInput::make('email_nfe')
                                    ->label('E-mail NF-e')
                                    ->placeholder('Informe o e-mail')
                                    ->required()
                                    ->email()
                                    ->maxLength(255),
                                Forms\Components\TextInput::make('website')
                                    ->label('Website')
                                    ->placeholder('Informe o website')
                                    ->url()
                                    ->columnSpan(2)
                                    ->maxLength(255),
                                Forms\Components\TextInput::make('responsible_name')
                                    ->label('Responsável')
                                    ->placeholder('Informe o responsável')
                                    ->columnSpan(2)
                                    ->maxLength(255),
                            ])->columns(4),
                        Forms\Components\Textarea::make('observation')
                            ->label('Observação')
                            ->placeholder('Informe a observação')
                            ->columnSpanFull(),
                        Forms\Components\TextInput::make('avatar_url')
                            ->maxLength(255),
                        Forms\Components\Toggle::make('is_active')
                            ->label('Ativo')
                            ->default(true)
                            ->hiddenOn(['create'])
                            ->required(),
                    ]),
            ]);
    }

    public static function table(Table $table): Table
    {
        return $table
            ->columns([
                Tables\Columns\TextColumn::make('document_number')
                    ->label('CNPJ')
                    ->searchable(),
                Tables\Columns\TextColumn::make('state_registration')
                    ->label('Inscrição Estadual')
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('name')
                    ->label('Razão Social')
                    ->searchable(),
                Tables\Columns\TextColumn::make('fantasy_name')
                    ->label('Nome Fantasia')
                    ->searchable(),
                Tables\Columns\TextColumn::make('responsible_name')
                    ->label('Responsável')
                    ->searchable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('mobile_number')
                    ->label('Celular')
                    ->searchable(),
                Tables\Columns\TextColumn::make('phone_number')
                    ->label('Telefone')
                    ->searchable(),
                Tables\Columns\TextColumn::make('email')
                    ->label('E-mail')
                    ->searchable(),
                Tables\Columns\TextColumn::make('email_nfe')
                    ->label('E-mail NF-e')
                    ->searchable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\IconColumn::make('is_active')
                    ->label('Ativo')
                    ->boolean(),
                Tables\Columns\TextColumn::make('created_at')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('updated_at')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
                Tables\Columns\TextColumn::make('deleted_at')
                    ->dateTime()
                    ->sortable()
                    ->toggleable(isToggledHiddenByDefault: true),
            ])
            ->filters([
                Tables\Filters\TrashedFilter::make(),
            ])
            ->actions([
                Tables\Actions\ViewAction::make(),
                Tables\Actions\EditAction::make(),
            ])
            ->bulkActions([
                Tables\Actions\BulkActionGroup::make([
                    Tables\Actions\DeleteBulkAction::make(),
                    Tables\Actions\ForceDeleteBulkAction::make(),
                    Tables\Actions\RestoreBulkAction::make(),
                ]),
            ]);
    }

    public static function getRelations(): array
    {
        return [
            //
        ];
    }

    public static function getPages(): array
    {
        return [
            'index'  => Pages\ListRegistries::route('/'),
            'create' => Pages\CreateRegistry::route('/create'),
            'view'   => Pages\ViewRegistry::route('/{record}'),
            'edit'   => Pages\EditRegistry::route('/{record}/edit'),
        ];
    }

    public static function getEloquentQuery(): Builder
    {
        return parent::getEloquentQuery()
            ->withoutGlobalScopes([
                SoftDeletingScope::class,
            ]);
    }
}
