<?php

namespace App\Services;

use App\Models\RegistryAccount;
use Filament\Facades\Filament;
use Illuminate\Support\Facades\DB;

class RegistryAccountService
{
    public function updateBalance(int $id, float $newAddedMoney = 0, float $newDebitMoney = 0, $description = null)
    {
        // Iniciar uma transação para garantir a integridade dos dados
        DB::beginTransaction();

        try {
            $account = RegistryAccount::find($id);

            if (! $account) {
                throw new \Exception('Account not found');
            }

            // Subtrai os valores antigos do saldo
            $currentBalance = $account->balance_money;
            $oldAddedMoney  = $account->added_money;
            $oldDebitMoney  = $account->debit_money;

            // Calcula o novo saldo para o registro atual
            $newBalance = $currentBalance - $oldAddedMoney + $newAddedMoney + $oldDebitMoney - $newDebitMoney;

            // Atualiza o registro atual
            $account->update([
                'added_money'   => $newAddedMoney,
                'debit_money'   => $newDebitMoney,
                'balance_money' => $newBalance,
                'description'   => $description,
            ]);

            // Atualiza os saldos dos registros subsequentes
            $this->updateSubsequentBalances($account->id);

            // Commit da transação
            DB::commit();

            return $account;
        } catch (\Exception $e) {
            // Rollback da transação em caso de erro
            DB::rollBack();

            throw $e;
        }
    }

    private function updateSubsequentBalances($startId)
    {
        $accounts = RegistryAccount::where('id', '>', $startId)->orderBy('id')->get();

        foreach ($accounts as $account) {
            $previousAccount = RegistryAccount::where('id', '<', $account->id)->orderBy('id', 'desc')->first();
            $previousBalance = $previousAccount ? $previousAccount->balance_money : 0;
            $newBalance      = $previousBalance + $account->added_money - $account->debit_money;

            $account->update(['balance_money' => $newBalance]);
        }
    }

    public function createAccount($data)
    {
        $addedMoney = $data['added_money'] ?? 0;
        $debitMoney = $data['debit_money'] ?? 0;

        $lastAccount    = RegistryAccount::orderBy('created_at', 'desc')->first();
        $currentBalance = $lastAccount ? $lastAccount->balance_money : 0;

        $newBalance = $currentBalance + $addedMoney - $debitMoney;

        $data['company_id']    = $data['company_id'] ?? Filament::getTenant()->id;
        $data['user_id']       = $data['user_id'] ?? auth()->user()->id;
        $data['added_money']   = $addedMoney;
        $data['debit_money']   = $debitMoney;
        $data['balance_money'] = $newBalance;
        $data['description']   = $data['description'] ?? 'Entrada de dinheiro';

        return RegistryAccount::create($data);
    }

    public function deleteAccount($id)
    {
        // Iniciar uma transação para garantir a integridade dos dados
        DB::beginTransaction();

        try {
            $account = RegistryAccount::find($id);

            if (! $account) {
                throw new \Exception('Account not found');
            }

            // Pegar o registro subsequente ao atual
            $nextAccounts = RegistryAccount::where('id', '>', $id)->orderBy('id')->get();

            // Excluir o registro atual
            $account->delete();

            // Atualizar os saldos dos registros subsequentes
            foreach ($nextAccounts as $nextAccount) {
                $previousAccount = RegistryAccount::where('id', '<', $nextAccount->id)->orderBy('id', 'desc')->first();
                $previousBalance = $previousAccount ? $previousAccount->balance_money : 0;
                $newBalance      = $previousBalance + $nextAccount->added_money - $nextAccount->debit_money;

                $nextAccount->update(['balance_money' => $newBalance]);
            }

            // Commit da transação
            DB::commit();
        } catch (\Exception $e) {
            // Rollback da transação em caso de erro
            DB::rollBack();

            throw $e;
        }
    }

    public function getCurrentBalance($registryId)
    {
        $lastAccount = RegistryAccount::where('registry_id', $registryId)->orderBy('created_at', 'desc')->first();

        return $lastAccount ? $lastAccount->balance_money : 0;
    }
}
